﻿using System;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Threading.Tasks;
using Aurora.Models.Media;
using Xamarin.Forms;
using Aurora.Design.Views.Main;

namespace Aurora.Design.Views
{

    public class BaseViewModel : INotifyPropertyChanged
    {
        private BaseMedia _baseMedia;
        public BaseViewModel()
        {
        }

        #region Player 

        /// <summary>
        /// Command event handler for player play button
        /// </summary>
        public virtual void OnPlayButtonExecute() { }
        public virtual bool CanPlayButtonExecute()
        {
            return true;
        }

        /// <summary>
        /// Command event handler for player next button
        /// </summary>
        public virtual void OnNextButtonExecute() { }
        public virtual bool CanNextButtonExecute()
        {
            return true;
        }

        /// <summary>
        /// Command event handler for player previous button
        /// </summary>
        public virtual void OnPreviousButtonExecute() { }
        public virtual bool CanPreviousButtonExecute()
        {
            return true;
        }

        /// <summary>
        /// Model for the currently playing music.
        /// </summary>
        /// <value></value>
        public BaseMedia Media
        {
            get
            {
                return this._baseMedia;
            }
            set
            {
                if (value != _baseMedia)
                {
                    _baseMedia = value;
                    if (this.SetPlayerMetadata != null)
                    {
                        SetPlayerMetadata.Invoke(value);
                    }
                }
            }
        }

        public SetPlayerMetadataDelegate SetPlayerMetadata { get; set; }

        public SetPlayerVisibleDelegate SetPlayerVisible { get; set; }

        #endregion Player

        #region Lifecycle
        public virtual Task OnActive() { return Task.FromResult<object>(null); }

        public virtual Task OnInactive() { return Task.FromResult<object>(null); }

        #endregion

        #region INotifyPropertyChanged Implementation
        public bool SetProperty<T>(ref T storage, T value, [CallerMemberName] string propertyName = null)
        {
            if (Object.Equals(storage, value))
                return false;

            storage = value;
            OnPropertyChanged(propertyName);
            return true;
        }

        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged([CallerMemberName] string propertyName = "")
        {
            if (PropertyChanged == null)
                return;

            PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion
    }
}
